/*
    Copyright (c) 2007-2009 FastMQ Inc.

    This file is part of 0MQ.

    0MQ is free software; you can redistribute it and/or modify it under
    the terms of the Lesser GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    0MQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Lesser GNU General Public License for more details.

    You should have received a copy of the Lesser GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __ZMQ_SUB_HPP_INCLUDED__
#define __ZMQ_SUB_HPP_INCLUDED__

#include <set>
#include <string>

#include "socket_base.hpp"
#include "yarray.hpp"

namespace zmq
{

    class sub_t : public socket_base_t
    {
    public:

        sub_t (class app_thread_t *parent_);
        ~sub_t ();

    protected:

        //  Overloads of functions from socket_base_t.
        void xattach_pipes (class reader_t *inpipe_, class writer_t *outpipe_);
        void xdetach_inpipe (class reader_t *pipe_);
        void xdetach_outpipe (class writer_t *pipe_);
        void xkill (class reader_t *pipe_);
        void xrevive (class reader_t *pipe_);
        int xsetsockopt (int option_, const void *optval_, size_t optvallen_);
        int xsend (zmq_msg_t *msg_, int flags_);
        int xflush ();
        int xrecv (zmq_msg_t *msg_, int flags_);
        bool xhas_in ();
        bool xhas_out ();

    private:

        //  Helper function to return one message choosed using
        //  fair queueing algorithm.
        int fq (zmq_msg_t *msg_, int flags_);

        //  Inbound pipes, i.e. those the socket is getting messages from.
        typedef yarray_t <class reader_t> in_pipes_t;
        in_pipes_t in_pipes;

        //  Number of active inbound pipes. Active pipes are stored in the
        //  initial section of the in_pipes array.
        in_pipes_t::size_type active;

        //  Index of the next inbound pipe to read messages from.
        in_pipes_t::size_type current;

        //  Number of active "*" subscriptions.
        int all_count;

        //  List of all prefix subscriptions.
        typedef std::multiset <std::string> subscriptions_t;
        subscriptions_t prefixes;

        //  List of all exact match subscriptions.
        subscriptions_t topics;

        sub_t (const sub_t&);
        void operator = (const sub_t&);
    };

}

#endif
